import java.awt.*;
import javax.swing.JPanel;
import java.util.*;

public class Labyrinthe extends JPanel {
    final int WINDOWS_SIZE = 300;
    final int GRID_SIZE = 10;
    int caseSize;
    Case[][] caseArray;
    final double pourcentageObstacle = 0.25;
    Case startCase;
    Case goalCase;


    public Labyrinthe() {
        this.setPreferredSize(new Dimension(WINDOWS_SIZE, WINDOWS_SIZE));
        this.caseArray = new Case[GRID_SIZE][GRID_SIZE];
        this.caseSize = WINDOWS_SIZE / GRID_SIZE;

        for (int i = 0; i < GRID_SIZE; i++) {
            for (int j = 0; j < GRID_SIZE; j++) {
                Vector<Case> voisines = new Vector<>();
              this.caseArray[i][j] = new Case(j * this.caseSize, i * this.caseSize, (j + 1) * this.caseSize,
                (i + 1) * this.caseSize, Math.random() <= this.pourcentageObstacle, i, j, voisines, (int) (Math.random()*6));

            }
        }

        for (int i = 0; i < GRID_SIZE; i++) {
            for (int j = 0; j < GRID_SIZE; j++) {
                // voisin de droite
                if (j < GRID_SIZE - 1) {
                    if (this.caseArray[i][j].isObstacle == this.caseArray[i][j + 1].isObstacle) {
                        this.caseArray[i][j].voisines.add(this.caseArray[i][j + 1]);
                    }
                }
                // voisin de gauche
                if (j > 0) {
                    if (this.caseArray[i][j].isObstacle == this.caseArray[i][j - 1].isObstacle) {
                        this.caseArray[i][j].voisines.add(this.caseArray[i][j - 1]);
                    }
                }
                // voisin du bas
                if (i < GRID_SIZE - 1) {
                    if (this.caseArray[i][j].isObstacle == this.caseArray[i + 1][j].isObstacle) {
                        this.caseArray[i][j].voisines.add(this.caseArray[i + 1][j]);
                    }
                }
                // voisin du haut
                if (i > 0) {
                    if (this.caseArray[i][j].isObstacle == this.caseArray[i - 1][j].isObstacle) {
                        this.caseArray[i][j].voisines.add(this.caseArray[i - 1][j]);
                    }
                }
            }
        }

        // position case de départ
        int s;
        s = (int) (Math.random() * (GRID_SIZE - 1));
        this.startCase = this.caseArray[s][0];
        // position case d'arrivée
        int g;
        g = (int) (Math.random() * (GRID_SIZE - 1));
        this.goalCase = this.caseArray[g][GRID_SIZE - 1];

        boolean cheminTrouve = this.pathFindAstar();
    }

    boolean pathFindAstar() {
        PriorityQueue<Case> openList = new PriorityQueue<>();
        Set<Case> closedList = new HashSet<>();
        int N = GRID_SIZE * GRID_SIZE;
        this.computeH();
        openList.add(startCase);

      while(!openList.isEmpty()){
        Case current = openList.poll();
        
        if (current == goalCase) {
          reconstructPath(current);
          return true;
        }
        closedList.add(current);
        for (Case neighbor : current.voisines){
          if (!closedList.contains(neighbor) && !neighbor.isObstacle){
            int tentative_gScore = current.g + current.poids;
            if (tentative_gScore < neighbor.g) {
              neighbor.cameFrom = current;
              neighbor.g = tentative_gScore;
              neighbor.f = tentative_gScore + neighbor.h;
            
               if (!openList.contains(neighbor)){
                 openList.add(neighbor);
               }
            }        
          }      
        }
      }
      return false; 
    }

  private void reconstructPath(Case current){
    while(current.cameFrom != null){
      current.isOnPath = true;
      current = current.cameFrom;
    }
  }

    void computeH() {
        for (int i = 0; i < GRID_SIZE; i++) {
            for (int j = 0; j < GRID_SIZE; j++) {
                this.caseArray[i][j].h = (int) (Math.abs(i - this.goalCase.positionI) + Math.abs(j - this.goalCase.positionJ));
            }
        }
    }

    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2D = (Graphics2D) g;
        for (int i = 0; i < GRID_SIZE; i++) {
            for (int j = 0; j < GRID_SIZE; j++) {
                g2D.drawRect(this.caseArray[i][j].x1, this.caseArray[i][j].y1, this.caseArray[i][j].x2 - this.caseArray[i][j].x1, this.caseArray[i][j].y2 - this.caseArray[i][j].y1);
                g.drawString(Integer.toString(this.caseArray[i][j].poids), this.caseArray[i][j].x1 +15, (this.caseArray[i][j].y1 +15));
            }
        }

        g.setColor(Color.green);
        g2D.fillRect(this.startCase.x1 + 2, this.startCase.y1 + 2, this.caseSize - 4, this.caseSize - 4);
        g.setColor(Color.red);
        g2D.fillRect(this.goalCase.x1 + 2, this.goalCase.y1 + 2, this.caseSize - 4, this.caseSize - 4);

      for(int i=0;i<GRID_SIZE;i++){
        for(int j=0;j<GRID_SIZE;j++){
            if(this.caseArray[i][j].isOnPath && this.caseArray[i][j] != this.goalCase){
              g.setColor(Color.red);
              g.drawString(Integer.toString(this.caseArray[i][j].poids), this.caseArray[i][j].x1 +15, (this.caseArray[i][j].y1 +15));
            }
          
          }
        }
    
    // ligne mur de droite
    for(int i=0;i<GRID_SIZE;i++){
      for(int j=0;j<GRID_SIZE;j++){
        if(this.caseArray[i][j].isObstacle){
          // vérifie si la case à droite n'est pas un obstacle
          if(j<GRID_SIZE-1){
            if(!this.caseArray[i][j+1].isObstacle){
              g2D.setColor(Color.black);
              g2D.setStroke(new BasicStroke(3));
              g2D.drawLine(this.caseArray[i][j].x2,this.caseArray[i][j].y1,this.caseArray[i][j].x2,this.caseArray[i][j].y2);
            }
          }
          // si la case est située à l'extrême droite
          else{
            g2D.setColor(Color.black);
            g2D.setStroke(new BasicStroke(3));
            g2D.drawLine(this.caseArray[i][j].x2,this.caseArray[i][j].y1,this.caseArray[i][j].x2,this.caseArray[i][j].y2);
          }
        }
      }
    }

    // ligne mur de gauche
    for (int i = 0; i < GRID_SIZE; i++) {
      for (int j = 0; j < GRID_SIZE; j++) {
          if (this.caseArray[i][j].isObstacle) {
            // vérifie si la case à gauche n'est pas un obstacle
            if(j>0){
              if (!this.caseArray[i][j - 1].isObstacle){
                g2D.setColor(Color.black);
                g2D.setStroke(new BasicStroke(3));
                g2D.drawLine(this.caseArray[i][j].x1, this.caseArray[i][j].y1, this.caseArray[i][j].x1, this.caseArray[i][j].y2);
            }
          }
          // si la case est située à l'extrême gauche
          else{
            g2D.setColor(Color.black);
            g2D.setStroke(new BasicStroke(3));
            g2D.drawLine(this.caseArray[i][j].x1, this.caseArray[i][j].y1, this.caseArray[i][j].x1, this.caseArray[i][j].y2);
          }
        }
    }
 }

    // ligne mur du bas
    for(int i=0;i<GRID_SIZE;i++){
      for(int j=0;j<GRID_SIZE;j++){
        if(this.caseArray[i][j].isObstacle){
          // vérifie si la case en bas n'est pas un obstacle
          if(i<GRID_SIZE-1){
            if(!this.caseArray[i+1][j].isObstacle){
              g2D.setColor(Color.black);
              g2D.setStroke(new BasicStroke(3));
              g2D.drawLine(this.caseArray[i][j].x1,this.caseArray[i][j].y2,this.caseArray[i][j].x2,this.caseArray[i][j].y2);
            }
          }
          // si la case est située tout en bas
          else{
            g2D.setColor(Color.black);
            g2D.setStroke(new BasicStroke(3));
            g2D.drawLine(this.caseArray[i][j].x1,this.caseArray[i][j].y2,this.caseArray[i][j].x2,this.caseArray[i][j].y2);
          }
        }
      }
    }

      


      
    // ligne mur du haut
    for(int i=0;i<GRID_SIZE;i++){
      for(int j=0;j<GRID_SIZE;j++){
        // vérifie si la case en haut n'est pas un obstacle
        if(this.caseArray[i][j].isObstacle){
          if(i>0){
            if(!this.caseArray[i-1][j].isObstacle){
              g2D.setColor(Color.black);
              g2D.setStroke(new BasicStroke(3));
              g2D.drawLine(this.caseArray[i][j].x1,this.caseArray[i][j].y1,this.caseArray[i][j].x2,this.caseArray[i][j].y1); 
            }
          }
          else{
            g2D.setColor(Color.black);
            g2D.setStroke(new BasicStroke(3));
            g2D.drawLine(this.caseArray[i][j].x1,this.caseArray[i][j].y1,this.caseArray[i][j].x2,this.caseArray[i][j].y1); 

   
                
              
        
          }
        }
      }
    }
      /*g2D.setPaint(Color.blue);
      g2D.drawLine(0,0,200,200);
      g2D.drawRect(10,10,50,50);

      g2D.setStroke(new BasicStroke(5));
      g.drawString("Dessin",100,50);

      ImageIcon icone = new ImageIcon("prune.png");
      Image image = icone.getImage();
      g.drawImage(image,60,60,null);

      g.setColor(Color.red);
      g.drawLine(0,0,100,200)
      g.drawRect(100,100,60,150);

      g.setColor(Color.orange);   
      g2D.drawOval(50,50,100,150);

      g.setColor(Color.black);
      g2D.fillOval(20,20,30,30);*/  
  }	
}


  


